!=====================================================!
!                                                     !
!     This work is distributed under CC_BY_NC_SA      !
!                                                     !
! Created by Ruben M. Cabezon and Domingo Garcia-Senz !
!               ruben.cabezon@unibas.ch               !
!               domingo.garcia@upc.edu                !
!                                                     !
!                        (2018)                       !
!                                                     !
!=====================================================!
!                                                     !
!               SPHYNX: conservemod.f90               !
!                                                     !
! Checks conservation laws.                           !
!=====================================================!

subroutine conservation
      
  USE parameters
      
  IMPLICIT NONE

  INTEGER i

  DOUBLE PRECISION dgrav,ax,ay,az
  DOUBLE PRECISION vmod2,ecin,eint,egrav,etot
  DOUBLE PRECISION acumlmx,acumlmy,acumlmz,acumamx,acumamy,acumamz
  DOUBLE PRECISION linmo,angmo

  call profile(0)

  if(flags)write(*,*) 'Conservation Laws'
  if(id.eq.0)open(3,file='conservelaws.d',position='append')
3 format(i10,10(1x,es17.10))


  ecin=0.d0
  eint=0.d0
  egrav=0.d0
  acumlmx=0.d0
  acumlmy=0.d0
  acumlmz=0.d0
  acumamx=0.d0
  acumamy=0.d0
  acumamz=0.d0


!$omp parallel private(i,ax,ay,az,dgrav) &
!$omp          reduction(+:acumlmx,acumlmy,acumlmz)&
!$omp          reduction(+:acumamx,acumamy,acumamz)&
!$omp          reduction(+:ecin,egrav,eint)
!$omp do schedule(static)
  do i=1,n
     acumlmx=acumlmx+v(i)*masa(i)
     acumlmy=acumlmy+v(i+n)*masa(i)
     acumlmz=acumlmz+v(i+n2)*masa(i)
     ax=a(i)-despl
     ay=a(i+n)-despl
     az=a(i+n2)-despl
     acumamx=acumamx+(ay*v(i+n2)-az*v(i+n))*masa(i)
     acumamy=acumamy+(az*v(i)-ax*v(i+n2))*masa(i)
     acumamz=acumamz+(ax*v(i+n)-ay*v(i))*masa(i)
     vmod2=v(i)*v(i)+v(i+n)*v(i+n)+v(i+n2)*v(i+n2)
     ecin=ecin+.5d0*masa(i)*vmod2
     dgrav=-masa(i)*g*.5d0*ugrav(i)
     egrav=egrav+dgrav
     eint=eint+u(i)*masa(i)
  enddo
!$omp end do
!$omp end parallel
  linmo=dsqrt(acumlmx**2+acumlmy**2+acumlmz**2)
  angmo=dsqrt(acumamx**2+acumamy**2+acumamz**2)
  etot=ecin+eint+egrav
  if(id.eq.0) then
     write(3,3) l,tt,ecin,eint,egrav,etot,linmo,angmo
     close(3)
  endif

  call profile(1)

  return
end subroutine conservation
