!=====================================================!
!                                                     !
!     This work is distributed under CC_BY_NC_SA      !
!                                                     !
! Created by Ruben M. Cabezon and Domingo Garcia-Senz !
!               ruben.cabezon@unibas.ch               !
!               domingo.garcia@upc.edu                !
!                                                     !
!                        (2018)                       !
!                                                     !
!=====================================================!
!                                                     !
!              SPHYNX: testparameters.f90             !
!                                                     !
! Checks correctness of parameters.                   !
!=====================================================!

 SUBROUTINE testparameters

    USE parameters
    
    IMPLICIT NONE
    
    integer status
    
    status=0
    if(iterini.ne.1.and.timeini.eq.0.d0) then
       status=1
       write(*,*) 'WARNING: This is a restart (iterini!=1) with timeini!=0'
    endif

    if(equalization.and.kernel.ne.2.and.kernel.ne.3) then
       status=2
       write(*,*) 'ERROR: Equalization must be used with sinc kernels (2 or 3)'
    endif
    
    if(pexp.eq.1.d0.and..not.(volstdprom)) then
       status=2
       write(*,*) 'ERROR: pexp should be < 1 if volstdprom=.false.'
    endif
    
    if(deltahindex.eq.0.d0.and.equalization) then
       status=2
       write(*,*) 'ERROR: Equalization is disabled if deltahindex=0'
    endif
    
    if(.not.timecour.and..not.timero.and..not.timeacc) then
       status=2
       write(*,*) 'ERROR: No time-step criteria selected'
    endif

    if(dim.ne.2.and.dim.ne.3) then
       status=2
       write(*,*) 'ERROR: wrong dimension selected (dim)'
    endif

    if(ncubes.ne.1.and.ncubes.ne.9.and.ncubes.ne.27) then
       status=2
       write(*,*) 'ERROR: wrong number of cubes for PBC (ncubes)'
    endif
    
    if(status.eq.0) then
       write(*,*) 'parameters testing .................. OK'
    else if(status.eq.1) then
       write(*,*) 'parameters testing .................. WARNING(s)!'
    else
       write(*,*) 'parameters testing .................. FAILED!'
       stop
    endif
    

    RETURN
  END SUBROUTINE testparameters
